#include <EEPROM.h>

const String EEPROM_Headder = "MIRO01";

void ReadSettings() {
  EEPROM.begin(128);
  if (EEPROM_CheckValid()) {
    THETA_RSSI_THRESHOLD = EEPROM_ReadInt(8);
    THETA_SSID           = EEPROM_ReadString(10,SSIDPASS_MAXLEN);
    THETA_Password       = EEPROM_ReadString(10+SSIDPASS_MAXLEN,SSIDPASS_MAXLEN);
  }
  EEPROM.end();
}

void WriteSettings() {
  EEPROM.begin(128);
  EEPROM_WriteString(EEPROM_Headder, 0, 8);
  EEPROM_WriteInt(THETA_RSSI_THRESHOLD,8);
  EEPROM_WriteString(THETA_SSID,10,SSIDPASS_MAXLEN);
  EEPROM_WriteString(THETA_Password,10+SSIDPASS_MAXLEN,SSIDPASS_MAXLEN);
  EEPROM.end();
}

void ClearSettings() {
  for (int i = 0; i < 128; i++)  EEPROM.write(i, 0); 
}

bool EEPROM_CheckValid() {
  String CheckValue = EEPROM_ReadString(0,8);
  return CheckValue.equals(EEPROM_Headder);
}

String EEPROM_ReadString(int pos,int len) {
  String result = "";

  for(int i=0;i<len;i++) {
    char ch = EEPROM.read(pos+i);
    if (ch == 0) break;
    result = result + ch;
  }
  return result;
}

int EEPROM_ReadInt(int pos) {
  return (int16_t)((EEPROM.read(pos)<<8) + EEPROM.read(pos+1));
}

void EEPROM_WriteString(String str,int pos,int len) {
  for(int i=0;i<len;i++) {
    if (i<str.length()) {
      EEPROM.write(pos+i,str.charAt(i));
    } else {
      EEPROM.write(pos+i,0);
    }
  }
}

void EEPROM_WriteInt(int dat,int pos) {
  EEPROM.write(pos  , (uint8_t)((dat >> 8) & 0xFF));
  EEPROM.write(pos+1, (uint8_t)( dat       & 0xFF));
}

