#include <ESP8266WiFi.h>
#include <EEPROM.h>


// ------------------------------------------------------------
//   Global
// ------------------------------------------------------------
#define   SHUTTER_PIN         (14)
#define   LED_PIN             (12)

#define   MODE_SHUTTER        (0)
#define   MODE_SCAN           (1)
#define   MODE_HTTPD          (2)

char      gnSettingMode         = 0;

#define   SSIDPASS_MAXLEN     (32)
String    THETA_SSID            = "";
String    THETA_Password        = "";
int       THETA_RSSI_THRESHOLD  = -50;

#define LED_ON      (HIGH)
#define LED_OFF     (LOW)

int Pattern_ON[]         = { 1,  LED_ON , 1000 };
int Pattern_OFF[]        = { 1,  LED_OFF, 1000 };
int Pattern_Connecting[] = { 2,  LED_ON ,   10,  LED_OFF, 1990 };
int Pattern_Connected[]  = { 2,  LED_ON ,   10,  LED_OFF,   90 };
int Pattern_Capture[]    = { 2,  LED_ON ,   10,  LED_OFF,  290 };
int Pattern_Pairing[]    = { 4,  LED_ON ,   10,  LED_OFF,  190, LED_ON ,   10,  LED_OFF,  790 };
int Pattern_Paired[]     = { 2,  LED_ON , 2000,  LED_OFF,60000 };
int Pattern_Server[]     = { 4,  LED_ON ,  250,  LED_OFF,  250, LED_ON ,   10,  LED_OFF,  490 };


// ------------------------------------------------------------
//   Setup
// ------------------------------------------------------------
void setup() {
  // I/Oを設定する
  pinMode( SHUTTER_PIN,  INPUT_PULLUP );
  pinMode( LED_PIN,      OUTPUT );
  digitalWrite( LED_PIN, HIGH );

  // Serial
  Serial.begin(115200);
  Serial.println("Initialize");

  // LED
  LED_setup();
  LED(Pattern_OFF);

  // 設定値を読む
  ReadSettings();
  Serial.print("THETA_RSSI_THRESHOLD:");
  Serial.println(THETA_RSSI_THRESHOLD);
  Serial.print("THETA_SSID:");
  Serial.println(THETA_SSID);
  Serial.print("THETA_Password:");
  Serial.println(THETA_Password);

  // 初期設定されていなかったら強制的にペアリングモード
  if (THETA_SSID.equals("")) {
    gotoPairingMode();
    return;
  }

  // ボタン押されている？
  if (digitalRead( SHUTTER_PIN ) == LOW) {
    LED(Pattern_ON);

    // 長押し確認
    unsigned long startTime = millis();
    bool isLongPress = false;
    while(digitalRead( SHUTTER_PIN ) == LOW) {
      delay(100);
      if ((millis()-startTime)>4000) {
        isLongPress = true;
        break;
      }
    }
  
    // 電源投入時ボタン長押し＝フルセッティングモード（httpd起動）
    if (isLongPress) {
      gotoServerMode();
    } else {
      gotoPairingMode();
    }
  } else {
    // 電源投入時ボタン押されていない＝シャッターボタンモード
    gotoShutterMode();
  }
}

void gotoShutterMode() {
  initialize_shutter();
  gnSettingMode   = MODE_SHUTTER;
}

void gotoPairingMode() {
  initialize_scan();
  gnSettingMode   = MODE_SCAN;
}

void gotoServerMode() {
  initialize_httpd();
  gnSettingMode   = MODE_HTTPD;  
}

// ------------------------------------------------------------
//   loop
// ------------------------------------------------------------
void loop() {
  if (gnSettingMode == MODE_SHUTTER) {
    loop_shutter();
  } else if (gnSettingMode == MODE_SCAN) {
    loop_scan();
  } else if (gnSettingMode == MODE_HTTPD) {
    loop_httpd();
  }
}



