// ------------------------------------------------------------
//   Global
// ------------------------------------------------------------
#define   THETA_IP            "192.168.1.1"
#define   THETA_PORT          (15740)

const char g_sClientName[]  = "THETA_Remote2";
const char g_sClientGUID[]  = {  0x39, 0x08, 0x93, 0xea, 0x8d, 0xd7, 0x44, 0xff, 0x95, 0xc7, 0x27, 0xa2, 0x20, 0xd5, 0x13, 0x36 };
/* 390893ea-8dd7-44ff-95c7-27a220d51336 */

#define PTPIP_PACK_START                   (8)

/* PTP codes (named by libgphoto2) */
#define PTP_OC_OpenSession                 (0x1002)
#define PTP_OC_CloseSession                (0x1003)
#define PTP_OC_InitiateCapture             (0x100E)
#define PTP_OC_TerminateOpenCapture        (0x1018)
#define PTP_OC_InitiateOpenCapture         (0x101C)
#define PTP_OC_GetDevicePropValue          (0x1015)
#define PTP_OC_SetDevicePropValue          (0x1016)
#define PTP_EC_ObjectAdded                 (0x4002)
#define PTP_EC_DevicePropChanged           (0x4006)
#define PTP_EC_StoreFull                   (0x400A)
#define PTP_EC_CaptureComplete             (0x400D)
#define PTP_DPC_ExposureBiasCompensation   (0x5010)
#define PTP_DPC_StillCaptureMode           (0x5013)
#define PTP_DPC_CaptureStatus              (0xD808)


/* ----------------------------------------------
    Initiate Capture
   ---------------------------------------------- */
/* EV shift: 2000,1700,1300,1000,700,300,0,-300,-700,-1000,-1300,-1700,-2000 */

int InitiateCapture(WiFiClient *nCommandSock, WiFiClient *nEventSock, int *nTransactionId, short evshift) {
  int aArgs[4];
  int nRes;
  unsigned char aPayload[2];
  unsigned char *aData;
  int nDataLen;
  int nHandle;
  int loop;
  int event_id,event_param[3];

  /* LED */
  Serial.print("InitiateCapture ("); Serial.print(evshift); Serial.print(")\n");
//  AirPen_LED(0);

  /* PTP_OC_SetDevicePropValue */
  PTPIP_Store_WORD(aPayload, evshift);
  aArgs[0] = PTP_DPC_ExposureBiasCompensation;
  nRes = PTPIP_Cmd_Request(nCommandSock,*nTransactionId++, PTP_OC_SetDevicePropValue, aArgs, 1, aPayload, 2);
  if (nRes<0) {
    Serial.print("PTP_OC_SetDevicePropValue Error\n");
    return -1;
  }
  nRes = PTPIP_Wait_Cmd_Response(nCommandSock, &aData, &nDataLen);
  if (aData != NULL) { free(aData); }
  if (nRes<0) {
    Serial.print("PTPIP_Wait_Cmd_Response Error\n");
    return -1;
  }

  /* PTP_OC_InitiateCapture */
  aArgs[0] = 0;
  aArgs[1] = 0;
  nRes = PTPIP_Cmd_Request(nCommandSock,*nTransactionId++, PTP_OC_InitiateCapture, aArgs, 2, NULL, 0);
  if (nRes<0) {
    Serial.print("PTPIP_Cmd_Request Error\n");
    return -1;
  }
  nRes = PTPIP_Wait_Cmd_Response(nCommandSock, &aData, &nDataLen);
  if (aData != NULL) { free(aData); }
  if (nRes<0) {
    Serial.print("PTPIP_Wait_Cmd_Response Error\n");
    return -1;
  }

  /* Wait PTP_EC_CaptureComplete */
  nHandle = 0;
  for(loop=0;loop<20;loop++) {
    event_id = 0;
    nRes = PTPIP_Wait_Event(nEventSock, 500, &event_id, event_param);
    if (event_id == PTP_EC_CaptureComplete) {
      /* CaptureComplete */
      Serial.print("CaptureComplete\n");
      break;
    } else if (event_id == PTP_EC_ObjectAdded) {
      /* Object Added */
      Serial.print("Object Added\n");
      nHandle = event_param[0];
      break;
    } else if (event_id == PTP_EC_StoreFull) {
      Serial.print("Store Full\n");
      nHandle = 0;
      break;
    }
  }

  /* PTP_OC_SetDevicePropValue */
  PTPIP_Store_WORD(aPayload, 0);
  aArgs[0] = PTP_DPC_ExposureBiasCompensation;
  nRes = PTPIP_Cmd_Request(nCommandSock,*nTransactionId++, PTP_OC_SetDevicePropValue, aArgs, 1, aPayload, 2);
  if (nRes<0) {
    Serial.print("PTP_OC_SetDevicePropValue Error\n");
    return -1;
  }
  nRes = PTPIP_Wait_Cmd_Response(nCommandSock, &aData, &nDataLen);
  if (aData != NULL) { free(aData); }
  if (nRes<0) {
    Serial.print("PTPIP_Wait_Cmd_Response Error\n");
    return -1;
  }

  /* LED */
//  AirPen_LED(1);

  return nHandle;
}


/* ----------------------------------------------
    Initiate or Terminate Capture (Recording)
   ---------------------------------------------- */

int OpenCapture(WiFiClient *nCommandSock, WiFiClient *nEventSock, int *nTransactionId, int bTerminate) {
  int aArgs[4];
  int nRes;
  unsigned char aPayload[2];
  unsigned char *aData;
  int nDataLen;
  int nHandle;
  int loop;
  int event_id,event_param[3];

  /* LED */
//  AirPen_LED(0);

  /* PTP_OC_InitiateOpenCapture or PTP_OC_TerminateOpenCapture */
  if (!bTerminate) {
    Serial.print("InitiateOpenCapture\n");
    aArgs[0] = 0;
    aArgs[1] = 0;
    nRes = PTPIP_Cmd_Request(nCommandSock,*nTransactionId++, PTP_OC_InitiateOpenCapture, aArgs, 2, NULL, 0);
  } else {
    Serial.print("TermninateOpenCapture\n");
    aArgs[0] = 0xFFFFFFFF;
    nRes = PTPIP_Cmd_Request(nCommandSock,*nTransactionId++, PTP_OC_TerminateOpenCapture, aArgs, 1, NULL, 0);
  }
  if (nRes<0) {
    Serial.print("PTPIP_Cmd_Request Error\n");
    return -1;
  }
  nRes = PTPIP_Wait_Cmd_Response(nCommandSock, &aData, &nDataLen);
  if (aData != NULL) { free(aData); }
  if (nRes<0) {
    Serial.print("PTPIP_Wait_Cmd_Response Error\n");
    return -1;
  }

  /* Wait PTP_EC_CaptureComplete */
  nHandle = 0;
  for(loop=0;loop<20;loop++) {
    event_id = 0;
    nRes = PTPIP_Wait_Event(nEventSock, 500, &event_id, event_param);
    if (event_id == PTP_EC_DevicePropChanged) {
      /* DevicePropChanged */
      Serial.print("DevicePropChanged\n");
      nHandle = 123; /* dummy for ESP8266 */
      break;
    } else if (event_id == PTP_EC_ObjectAdded) {
      /* Object Added */
      Serial.print("Object Added\n");
      nHandle = event_param[0];
      break;
    } else if (event_id == PTP_EC_StoreFull) {
      Serial.print("Store Full\n");
      nHandle = 0;
      break;
    }
  }

  /* LED */
//  AirPen_LED(1);

  return nHandle;
}


/* ----------------------------------------------
    GetDevicePropValue
   ---------------------------------------------- */

int GetDevicePropValueWord(WiFiClient *nCommandSock, WiFiClient *nEventSock, int *nTransactionId, int nPropCode) {
  int aArgs[4];
  int nRes;
  unsigned char aPayload[2];
  unsigned char *aData;
  int nDataLen;
  int nResult;

  /* PTP_OC_GetDevicePropValue */
  aArgs[0] = nPropCode;
  nRes = PTPIP_Cmd_Request(nCommandSock,*nTransactionId++, PTP_OC_GetDevicePropValue, aArgs, 1, aPayload, 0);
  if (nRes<0) {
    Serial.print("PTP_OC_GetDevicePropValue Error\n");
    return -1;
  }
  nRes = PTPIP_Wait_Cmd_Response(nCommandSock, &aData, &nDataLen);
  if ((nDataLen<2) || (aData == NULL) || (nRes<0)) {
    if (aData != NULL) { free(aData); }
    Serial.print("PTPIP_Wait_Cmd_Response Error\n");
    return -1;
  }
  nResult = PTPIP_Get_WORD(aData);
  free(aData);
  return nResult;
}




/* ----------------------------------------------
    Init_Command
   ---------------------------------------------- */

int PTPIP_Init_Command_Request(WiFiClient *sock) {
  unsigned char aPayload[250];
  char *name;
  int i,len,res;

  /* GUID */
  for(i=0;i<16;i++) {
    aPayload[8+i] = g_sClientGUID[i];
  }
  /* Client Name */
  name = (char *)g_sClientName;
  i = 24;
  while(*name) {
    aPayload[i++] = *name++;
    aPayload[i++] = 0;
    if (i>=(sizeof(aPayload)-6)) {
      aPayload[i++] = 0;
      aPayload[i++] = 0;
      break;
    }
  }
  PTPIP_Store_DWORD(aPayload+i, 1); i+=4;

  len = i;
  PTPIP_pack_command(aPayload, 1, len);
  /* Send */
  res = writeClient(sock, aPayload, len);
  if (res<0) { return -1; }
  return 0;
}

int PTPIP_Wait_Init_Command_Ack(WiFiClient *sock,int *session_id) {
  unsigned char aPayload[250];
  int res,cmd;

  res = PTPIP_Recv_Response(sock, &cmd, aPayload, sizeof(aPayload));
  if (res < 0) { return -1; }
  if (cmd != 2) { return -1; }
  *session_id = PTPIP_Get_DWORD(aPayload);
  return 0;
}

/* ----------------------------------------------
    Init_Event
   ---------------------------------------------- */

int PTPIP_Init_Event_Request(WiFiClient *sock, int session_id) {
  unsigned char aPayload[16];
  int len,res;

  len = 12;
  PTPIP_Store_DWORD(aPayload+8, session_id);
  PTPIP_pack_command(aPayload, 3, len);
  /* Send */
  res = writeClient(sock, aPayload, len);
  if (res<0) { return -1; }
  return 0;
}

int PTPIP_Wait_Init_Event_Ack(WiFiClient *sock) {
  unsigned char aPayload[250];
  int res,cmd;

  res = PTPIP_Recv_Response(sock, &cmd, aPayload, sizeof(aPayload));
  if (res < 0) { return -1; }
  if (cmd != 4) { return -1; }
  return 0;
}


/* ----------------------------------------------
    Wait_Event
   ---------------------------------------------- */
/* event_param ... array[3] of int */

int PTPIP_Wait_Event(WiFiClient *sock, int wait, int *event_id, int event_param[]) {
  int n;
  unsigned char aPayload[250];
  int res,cmd,transaction_id,i;

  waitForPacket(sock, wait);
  if (sock->available()>0) {
    res = PTPIP_Recv_Response(sock, &cmd, aPayload, sizeof(aPayload));
    if (res < 0) { return -1; }
    if (cmd != 8) { return -1; }

    *event_id      = PTPIP_Get_WORD(aPayload);
    transaction_id = PTPIP_Get_DWORD(aPayload+2);
    res -= 6;
    i   = 0;
    while((res>0) && (i<3)) {
      event_param[i] = PTPIP_Get_DWORD(aPayload+6+i*4);
      res -= 4;
      i++;
    }
    return 0;
  } else {
    return -1;
  }
}



/* ----------------------------------------------
    Cmd_Request
   ---------------------------------------------- */

int PTPIP_Cmd_Request(WiFiClient *sock, int transaction_id, int cmd, int arg[], int arglen, unsigned char *payload, int payloadlen) {
  unsigned char aPayload[250];
  int len,i,res,packetlen;

  len = PTPIP_PACK_START;
  PTPIP_Store_DWORD(aPayload+len, 1); len += 4;
  PTPIP_Store_WORD(aPayload+len, cmd); len += 2;
  PTPIP_Store_DWORD(aPayload+len, transaction_id); len += 4;
  for(i=0;i<arglen;i++) {
    PTPIP_Store_DWORD(aPayload+len, arg[i]); len += 4;
    if (len>=sizeof(aPayload)) {
      return -1;
    }
  }
  PTPIP_pack_command(aPayload, 6, len);
  /* Send */
  res = writeClient(sock, aPayload, len);
  if (res<0) { return -1; }

  /* No payload */
  if ((payload == NULL) || (payloadlen==0)) { return 0; }

  /* Start Data Packet */
  len = PTPIP_PACK_START;
  PTPIP_Store_DWORD(aPayload+len, transaction_id); len += 4;
  PTPIP_Store_DWORD(aPayload+len, payloadlen); len += 4;
  PTPIP_Store_DWORD(aPayload+len, 0); len += 4;
  PTPIP_pack_command(aPayload, 9, len);
  /* Send */
  res = writeClient(sock, aPayload, len);
  if (res<0) { return -1; }

  /* Data Packet */
  while(payloadlen>0) {
    packetlen = payloadlen;
    if (packetlen>200) { packetlen = 200; }

    len = PTPIP_PACK_START;
    PTPIP_Store_DWORD(aPayload+len, transaction_id); len += 4;
    memcpy(aPayload+len, payload, packetlen); len += packetlen;
    payloadlen -= packetlen;
    payload    += packetlen;
    if (payloadlen != 0) {
      /* Data Packet */
      PTPIP_pack_command(aPayload, 10, len);
    } else {
      /* End Data Packet */
      PTPIP_pack_command(aPayload, 12, len);
    }
    /* Send */
    res = writeClient(sock, aPayload, len);
    if (res<0) { return -1; }
  }

  return 0;
}

int PTPIP_Wait_Cmd_Response(WiFiClient *sock, unsigned char **data, int *data_len) {
  unsigned char aPayload[250];
  int res,cmd,result;
  int transaction_id, temp_transaction_id, data_payload_size, buffer_size;
  unsigned char *buffer,*buffer_ptr;

  buffer      = NULL;
  buffer_ptr  = NULL;
  buffer_size = 0;
  result      = -1;

  res = PTPIP_Recv_Response(sock, &cmd, aPayload, sizeof(aPayload));
  if (res < 0) { goto wait_cmd_response_end; }

  /* with Data? */
  if (cmd == 9) {
    /* Start Data Packet */
    transaction_id    = PTPIP_Get_DWORD(aPayload);
    data_payload_size = PTPIP_Get_DWORD(aPayload+4);
    buffer      = (unsigned char *)malloc(data_payload_size);
    if (buffer == NULL) { goto wait_cmd_response_end; }
    buffer_ptr  = buffer;
    buffer_size = data_payload_size;
    
    /* continuous */
    while(data_payload_size>0) {
      res = PTPIP_Recv_Response(sock, &cmd, aPayload, sizeof(aPayload));
      if (res < 0) { goto wait_cmd_response_end; }
      if ((cmd != 10) && (cmd != 12)) {
        /* not Data_Packet and End_Data_Packet */
        goto wait_cmd_response_end;
      }
      
      temp_transaction_id = PTPIP_Get_DWORD(aPayload);
      res = res - 4;  /* payload length */
      if (transaction_id != temp_transaction_id) { goto wait_cmd_response_end; }
      if ((res>data_payload_size) || (res<=0))   { goto wait_cmd_response_end; }
      memcpy(buffer_ptr, aPayload+4, res);
      buffer_ptr        += res;
      data_payload_size -= res;
      if (cmd == 12) { break; } /* End_Data_Packet */
    }

    /* cmd response */
    res = PTPIP_Recv_Response(sock, &cmd, aPayload, sizeof(aPayload));
    if (res < 0) { goto wait_cmd_response_end; }
  }
  
  /* Response? */
  if (cmd == 7) {
    /* Cmd Response */
    transaction_id    = PTPIP_Get_DWORD(aPayload);
    /* finish */
    result = 0;
  }
wait_cmd_response_end:
  if ((result<0) && (buffer != NULL)) {
    free(buffer);
    buffer = NULL;
  } else {
    *data     = buffer;
    *data_len = buffer_size;
  }

  return result;
}


/* ----------------------------------------------
    PTPIP utils
   ---------------------------------------------- */

long PTPIP_Get_DWORD(unsigned char *ptr) {
  return (ptr[0] | (ptr[1] << 8) | (ptr[2] << 16) | (ptr[3] << 24));
}

short PTPIP_Get_WORD(unsigned char *ptr) {
  return (ptr[0] | (ptr[1] << 8));
}

void PTPIP_Store_WORD(unsigned char *ptr, short data) {
  ptr[ 0] = ( data        & 0xFF);
  ptr[ 1] = ((data >>  8) & 0xFF);
}

void PTPIP_Store_DWORD(unsigned char *ptr, long data) {
  ptr[ 0] = ( data        & 0xFF);
  ptr[ 1] = ((data >>  8) & 0xFF);
  ptr[ 2] = ((data >> 16) & 0xFF);
  ptr[ 3] = ((data >> 24) & 0xFF);
}

void PTPIP_pack_command(unsigned char *payload, int command, int length) {
  /* length */
  PTPIP_Store_DWORD(payload  , length);
  /* Command */
  PTPIP_Store_DWORD(payload+4, command);
}

int PTPIP_Recv_Response(WiFiClient *sock, int *cmd, unsigned char *payload, int payload_len) {
  int len,res;
  if (payload_len<4) { return -1; }
  /* packet length */
  res = readClient(sock, payload, 4);
  if (res<4) { return -1; }
  len = PTPIP_Get_DWORD(payload);
  /* command */
  res = readClient(sock, payload, 4);
  if (res<4) { return -1; }
  *cmd = PTPIP_Get_DWORD(payload);
  if (len==8) { return 0; }
  if (payload_len<(len-8)) { return -1; }
  res = readClient(sock, payload, (len-8));
  if (res<(len-8)) { return -1; }
  return (len-8);
}

// ------------------------------------------------------------
//   socket → arduino 移植サポート
// ------------------------------------------------------------

int writeClient(WiFiClient *sock, unsigned char *payload, int len) {
  int res = 0;
/*  
  Serial.print("Write:[ ");
  for(int i=0;i<len;i++) {
    if (payload[i]<0x10) { Serial.print("0"); }
    Serial.print(payload[i],HEX); Serial.print(" ");
  }
  Serial.print("]\n");
*/
  res = sock->write((uint8_t *)payload, len);
  delay(1);
  return res;
}

int readClient(WiFiClient *sock, unsigned char *payload, int len) {
  waitForPacket(sock, 10000);
  int rlen;

  rlen = sock->readBytes((uint8_t *)payload, len);
/* 
  Serial.print("Read :[ ");
  for(int i=0;i<rlen;i++) {
    if (payload[i]<0x10) { Serial.print("0"); }
    Serial.print(payload[i],HEX); Serial.print(" ");
  }
  Serial.print("]\n");
*/
  return rlen;  
}

void waitForPacket(WiFiClient *sock,int timeout) {
  unsigned long startMillis = millis();
  while((millis()-startMillis)<timeout) {
    if (sock->available()>0) break;
    delay(1);
  }
}

