// ------------------------------------------------------------
//   HTTPDセッティングモード
// ------------------------------------------------------------
#include <ESP8266WiFi.h>
#include <DNSServer.h>
#include <ESP8266WebServer.h>

const byte DNS_PORT = 53;
IPAddress apIP(192, 168, 1, 1);
DNSServer dnsServer;
ESP8266WebServer webServer(80);

void initialize_httpd() {
  LED(Pattern_Server);
  Serial.println("HTTPD mode...");
  //
  WiFi.mode(WIFI_AP);
  WiFi.softAPConfig(apIP, apIP, IPAddress(255, 255, 255, 0));
  WiFi.softAP("RemoteTHETA");

  dnsServer.setTTL(60);
  dnsServer.setErrorReplyCode(DNSReplyCode::ServerFailure);
  dnsServer.start(DNS_PORT, "*", apIP);

  // simple HTTP server to see that DNS server is working
  webServer.onNotFound([]() {
    String message = "file not found!\n\n";
    message += "URI: ";
    message += webServer.uri();

    webServer.send(404, "text/plain", message);
  });
  webServer.on("/", HTTP_GET, printTop);
  webServer.on("/setparam", HTTP_POST, setParam);
  webServer.on("/setparam", HTTP_GET, setParam);
  webServer.begin();
}

void loop_httpd() {
  dnsServer.processNextRequest();
  webServer.handleClient();
}



String htmlContent;

void printTop() {
  htmlContent = "";
  htmlContent = htmlContent +"<html>";
  htmlContent = htmlContent +"<head><title>RemoteTHETA Advanced Settings</title></head>";
  htmlContent = htmlContent +"<body>";
  htmlContent = htmlContent +"RemoteTHETA Settings<br/>";
  htmlContent = htmlContent +"<hr>";
  htmlContent = htmlContent +"<form action='/setparam' method=post>";
  htmlContent = htmlContent +"THETA_SSID : <input type=text name='THETA_SSID' value='"+THETA_SSID+"' size=32><br />";
  htmlContent = htmlContent +"THETA_Password : <input type=text name='THETA_Password' value='"+THETA_Password+"' size=32><br />";
  htmlContent = htmlContent +"<hr>";
  htmlContent = htmlContent +"Pairing RSSI Threshold : <input type=text name='THETA_RSSI_THRESHOLDA_Password' value='"+String(THETA_RSSI_THRESHOLD)+"' size=16><br />";
  htmlContent = htmlContent +"<input type=submit value='Set'><input type=submit name='RESET' value='Reset to default'>";
  htmlContent = htmlContent +"</form>";
  htmlContent = htmlContent +"<hr>";
  htmlContent = htmlContent +"(MIRO/@MobileHackerz)";
  htmlContent = htmlContent +"</body>";

  webServer.sendHeader("Connection", "close");
  webServer.sendHeader("Access-Control-Allow-Origin", "*");
  webServer.send(200, "text/html", htmlContent);
}

void setParam() {
  bool isUpdate = false;
  if (webServer.hasArg("THETA_SSID")) {
    THETA_SSID = webServer.arg("THETA_SSID");
    isUpdate = true;
  }
  if (webServer.hasArg("THETA_Password")) {
    THETA_Password = webServer.arg("THETA_Password");
    isUpdate = true;
  }
  if (webServer.hasArg("THETA_RSSI_THRESHOLD")) {
    THETA_RSSI_THRESHOLD = webServer.arg("THETA_RSSI_THRESHOLD").toInt();
    if (THETA_RSSI_THRESHOLD>0) { THETA_RSSI_THRESHOLD = -THETA_RSSI_THRESHOLD; }
    isUpdate = true;
  }
  if (webServer.hasArg("RESET")) {
    THETA_SSID = "";
    THETA_Password = "";
    THETA_RSSI_THRESHOLD = -50;
    isUpdate = true;
  }
  if (isUpdate) {
    WriteSettings();
  }  
  
  webServer.sendHeader("Connection", "close");
  webServer.sendHeader("Access-Control-Allow-Origin", "*");
  webServer.sendHeader("Location", "/");
  
  webServer.send(302, "text/html", "<html>go to <a href='/'>here</a></html>");
}

