// ------------------------------------------------------------
//   シャッターモード
// ------------------------------------------------------------
WiFiClient nCommandSock;
WiFiClient nEventSock;
short nEvshift;
int nTransactionId;
char bIsMovie;
int nSessionId;
char ssid[SSIDPASS_MAXLEN],pass[SSIDPASS_MAXLEN];

void initialize_shutter() {
  int nRes;
  int aArgs[4];
  unsigned char *aData;
  int nDataLen;

  LED(Pattern_Connecting);
  Serial.println("Shutter mode...");

  //WiFiクライアントモード設定
  WiFi.mode(WIFI_STA);

  //WiFi接続
  Serial.println();
  Serial.println();
  Serial.println("Connecting...");

  THETA_SSID.toCharArray(ssid,SSIDPASS_MAXLEN);
  THETA_Password.toCharArray(pass,SSIDPASS_MAXLEN);
  WiFi.begin(ssid,pass);
  
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }

  LED(Pattern_Connected);
  Serial.println("");
  Serial.println("WiFi connected");  
  Serial.println("IP address: ");
  Serial.println(WiFi.localIP());
  WiFi.printDiag(Serial);

  /* Init_Command */
  if (!nCommandSock.connect(THETA_IP, THETA_PORT)) {
    Serial.println("PTPIP_Open_Connection Error\n");
    ESP.restart();
  }
  nRes = PTPIP_Init_Command_Request(&nCommandSock);
  if (nRes<0) {
    Serial.println("PTPIP_Init_Command_Request Error\n");
    CommandSock_Close();
  }
  nRes = PTPIP_Wait_Init_Command_Ack(&nCommandSock,&nSessionId);
  if (nRes<0) {
    Serial.println("PTPIP_Wait_Init_Command_Ack Error\n");
    CommandSock_Close();
  }

  /* Init_Event */
  if (!nEventSock.connect(THETA_IP, THETA_PORT)) {
    Serial.println("PTPIP_Open_Connection Error\n");
    ESP.restart();
  }
  nRes = PTPIP_Init_Event_Request(&nEventSock,nSessionId);
  if (nRes<0) {
    Serial.println("PTPIP_Init_Event_Request Error\n");
    EventSock_Close();
  }
  nRes = PTPIP_Wait_Init_Event_Ack(&nEventSock);
  if (nRes<0) {
    Serial.println("PTPIP_Wait_Init_Event_Ack Error\n");
    EventSock_Close();
  }
  nTransactionId = 0;

  /* PTP_OC_OpenSession */
  aArgs[0] = nSessionId;
  nRes = PTPIP_Cmd_Request(&nCommandSock,nTransactionId++, PTP_OC_OpenSession, aArgs, 1, NULL, 0);
  if (nRes<0) {
    Serial.println("PTPIP_Cmd_Request Error\n");
    EventSock_Close();
  }
  nRes = PTPIP_Wait_Cmd_Response(&nCommandSock, &aData, &nDataLen);
  if (aData != NULL) { free(aData); }
  if (nRes<0) {
    Serial.println("PTPIP_Wait_Cmd_Response Error\n");
    EventSock_Close();
  }
  
  /* Check movie mode */
  nRes = GetDevicePropValueWord(&nCommandSock, &nEventSock, &nTransactionId, PTP_DPC_StillCaptureMode);
  if (nRes<0) {
    Serial.println("GetDevicePropValueWord Error\n");
    EventSock_Close();
  }
  bIsMovie = ((nRes == 0x0000) || (nRes == 0x0003));
  if (bIsMovie) {
    Serial.println("[Movie mode]\n");
  } else {
    Serial.println("[Still mode]\n");
  }

  LED(Pattern_ON);
}




void loop_shutter() {
  int nRes;
  char bIsRecording;

  /* Wait Trigger */
  if (digitalRead( SHUTTER_PIN ) != LOW) { return ; }

  LED(Pattern_Capture);
  if (!bIsMovie) {
    /* Still mode (InitiateCapture) */
    nCommandSock.flush();
    nEventSock.flush();
    nRes = InitiateCapture(&nCommandSock, &nEventSock, &nTransactionId,  nEvshift);
    if (nRes<0) {
      Serial.println("InitiateCapture Error\n");
      EventSock_Close();
    }
  } else {
    /* Movie mode (OpenCapture or TerminateOpenCapture) */
    if (nEvshift != 0) { return; }

    /* Check recording */
    nCommandSock.flush();
    nEventSock.flush();
    nRes = GetDevicePropValueWord(&nCommandSock, &nEventSock, &nTransactionId, PTP_DPC_CaptureStatus);
    if (nRes<0) {
      Serial.println("GetDevicePropValueWord Error\n");
      EventSock_Close();
    }
    bIsRecording = (nRes != 0x0000);
  
    if (!bIsRecording) {
      Serial.println("[Start record]\n");
    } else {
      Serial.println("[Stop record]\n");
    }
    nCommandSock.flush();
    nEventSock.flush();
    nRes = OpenCapture(&nCommandSock, &nEventSock, &nTransactionId, bIsRecording);
    if (nRes<0) {
      Serial.println("OpenCapture Error\n");
      EventSock_Close();
    }
    bIsRecording = !bIsRecording;
  }
  LED(Pattern_ON);

  /* Wait Trigger Up*/
  while (digitalRead( SHUTTER_PIN ) == LOW) { delay(1); }
}

void CommandSock_Close() {
  AirPen_LED(0);
  nCommandSock.stop();
  delay(1000);
  ESP.restart();
}

void EventSock_Close() {
  AirPen_LED(0);
  nCommandSock.stop();
  nEventSock.stop();
  delay(1000);
  ESP.restart();
}



void AirPen_LED(int led) {
}

